#' @title Refine the map
#'
#' @description Refine the axis labels, map projection, and theme to
#' improve the format of the map
#'
#' @param projection
#' Map projection. See the documentation for ggplot2 function coord_map.
#'
#' @param ...
#' Parameters for the map projection, if any. See the documentation for
#' ggplot function coord_map.
#'
#' @param longLimits
#' Limits for the longitude (in decimal degrees).
#'
#' @param latLimits
#' Limits for the latitude (in decimal degrees).
#'
#' @return
#' A list comprising ggplot objects of types scale_x_continuous,
#' scale_y_continous, coord_map, theme_bw, and theme.
#'
#' @export
#'
Refine_Map <- function(projection, ...,
                       longLimits = NULL,
                       latLimits = NULL) {
  return(list(ggplot2::scale_x_continuous(name = "Longitude, in degrees",
                                          limits = longLimits),
              ggplot2::scale_y_continuous(name = "Latitude, in degrees",
                                          limits = latLimits),
              ggplot2::coord_map(projection, ...),
              ggplot2::theme_bw(),
              ggplot2::theme(panel.grid.major = ggplot2::element_blank(),
                             panel.grid.minor = ggplot2::element_blank())))
}

#' @title Refine the legend
#'
#' @description Refine the legend to improve the format of the map
#'
#' @param legend.position
#' Position of the legend. See Wickham (2016, p. 120-122)
#'
#' @param legend.justification
#' Justification of the legend. See Wickham (2016, p. 120-122)
#'
#' @param legend.direction
#' Orientation of the legend. Either "vertical" or "horizontal".
#'
#' @return
#' A ggplot object of type theme.
#'
#' @references
#' Wickham, H., 2016, ggplot2-Elegant graphics for data analysis: Springer
#' International Publishing.
#'
#' @export
#'
Refine_Legend <- function(legend.position,
                          legend.justification,
                          legend.direction = "vertical"){

  return(ggplot2::theme(legend.position = legend.position,
                        legend.justification = legend.justification,
                        legend.direction = legend.direction,
                        legend.text = ggplot2::element_text(size = 6),
                        legend.title = ggplot2::element_text(size = 6)))
}


#' @title Add a path
#'
#' @description Add a path to a base map.
#'
#' @param pathData
#' R-language dataframe containing the locations of points along the path
#' (for example, end points of a line or vertices of polygon).
#' Columns "long" and "lat" list, respectively, the longitude and latitude of
#' points.
#'
#' @param color
#' The color of the path.
#'
#' @param ...
#' Additional arguments to ggplot2 function geom_path.
#'
#' @return
#' A ggplot object of type geom_path.
#'
#' @export
#'
Add_Path <- function(pathData, color = "black", ...){
  return(ggplot2::geom_path(ggplot2::aes(x = long, y = lat),
                              data = pathData,
                              color = color, ...))
}

#' @title Add an arc of a great circle
#'
#' @description Add an arc of a great circle to a base map.
#'
#' @param pathData
#' R-language dataframe containing the locations of the two endpoints of the
#' path. Each row of the dataframe pertains to one endpoint.
#' Columns "long" and "lat" list, respectively, the longitude and latitude of
#' the endpoints.
#'
#' @param color
#' The color of the path.
#'
#' @param nPoints
#' Number of interpolated points that will define the arc.
#'
#' @param ...
#' Additional arguments to ggplot2 function geom_path.
#'
#' @return
#' A ggplot object of type geom_path.
#'
#' @export
#'
Add_GreatCircleArc <- function(pathData, color = "black",
                                nPoints = 50, ...){

  points <- geosphere::gcIntermediate(pathData[1, ], pathData[2, ], n = nPoints)
  df <- tibble::as_tibble(points)
  colnames(df) <- c("long", "lat")

  return(Add_Path(df, color = color, ...))
}

#' @title Add points
#'
#' @description Add points to a base map.
#'
#' @param pointData
#' R-language dataframe containing the point locations. Columns "long" and "lat"
#' list, respectivley, the longitude and latitude of points.
#'
#' @param shape
#' The plot symbol used for the points.
#'
#' @param fraction
#' Fraction of the points that are plotted. The range of fraction is
#'      (0,1]. The default is 1, which means that all points are plotted.
#'
#' @param fill
#' Color of the inside of the plot symbol
#'
#' @param color
#' Color of the edge of the plot symbol
#'
#' @param size
#' Size of the plot symbol
#'
#' @return
#' A ggplot object of type geom_point.
#'
#' @export
#'
Add_Points <- function(pointData, shape = ".",
                       fraction = 1L, fill = "black",
                       color = fill, size = 1.0){

  n <- nrow(pointData)
  indices <- sample.int(n, size = as.integer(n * fraction))

  return(ggplot2::geom_point(ggplot2::aes(x = long, y = lat),
                             data = pointData[indices, , drop = FALSE],
                             shape = shape,
                             size = size,
                             fill = fill,
                             color = color))
}

#' @title Add marked points
#'
#' @description Add marked points to a base map.
#'
#' @param markedPointData
#' R-language dataframe containing the point locations and the quantity
#' that is plotted. Columns "long" and "lat" list, respectively,
#' the longitude and latitude of points.
#'
#' @param quantity
#' Character string with the name of the quantity that is plotted. It
#'       must be the name of a column in markedPointData.
#'
#' @param legendtitle
#' Character string with the legend title.
#'
#' @param values
#' R-valued vector containing the quantity values that are associated with the
#' color palette. This argument is used to scale the palette, which may improve
#' the format of the map.
#'
#' @param isColorContinuous
#' Logical indicator specifying whether the color scale for the marked points is
#' continuous. See Wickham (2016, p. 134-140)
#'
#' @param colors
#' Color palette used to plot the points.
#'
#' @param shape
#' The plot symbol used for the points.
#'
#' @param size
#' The size of the plot symbol.
#'
#' @param ...
#' Additional arguments to ggplot2 function scale_colour_gradientn.
#'
#' @return
#' A list comprising ggplot objects of types geom_point and either
#' scale_colour_gradientn or labs.
#'
#' @references
#' Wickham, H., 2016, ggplot2-Elegant graphics for data analysis: Springer
#' International Publishing.
#'
#' @export
#'
Add_Marked_Points <- function(markedPointData,
                              quantity,
                              legendtitle,
                              values,
                              isColorContinuous = TRUE,
                              colors = grDevices::terrain.colors(12),
                              shape = 15, size = 1.0, ...){

  return(list(
    ggplot2::geom_point(ggplot2::aes_(x = as.name("long"),
                                      y = as.name("lat"),
                                      colour = as.name(quantity)),
                        data = markedPointData,
                        shape = shape, size = size),
    if(isColorContinuous){
      ggplot2::scale_colour_gradientn(colors = colors,
                                      name = legendtitle,
                                      values = values,
                                      ...)
    } else {
      ggplot2::labs(color = legendtitle)
    }))
}
